import cv2 as cv
import numpy as np
import math


def ndaToTupleInt(arr):
    b = arr.astype(int)
    res = tuple(b)
    return res



# tar inn boks koordinater, gir ut lengde på alle sider og sidenes start og slutt koordinat.
# Kan gi avrundingsfeil på 1 px.
def findLines(box):
    n, d = np.shape(box)
    lines = []
    for i in np.arange(0, (n - 1)):
        p1 = np.array([box[i][0], box[i][1]])
        p2 = np.array([box[i + 1][0], box[i + 1][1]])
        l = cv.norm(p2 - p1)  # Regner ut lengde mellom punkter
        lines.append([round(l), p1, p2])
    p1 = np.array([box[0][0], box[0][1]])
    p2 = np.array([box[n - 1][0], box[n - 1][1]])
    l = cv.norm(p2 - p1)
    lines.append([round(l), p1, p2])

    lines = np.array(lines)

    return lines



# pos (0,0) er venstre/nederste side av korteste side som er lengst opp/venstre på bilde.


# Funksjonen om regner ut hvor alle POI skal være.
# Finner først gjørneposisjon og vinkel til rektangel,
def pathPoint(box, aks, diameter, overlapp, smallObjects):
    def calcPoint(cornPos, angle, pos):
        cornPos = np.asarray(cornPos, dtype=np.int)
        # Regner ut rotasjonsmatrise
        rotMat = cv.getRotationMatrix2D((0, 0), angle, 1.0)
        rot2D = np.delete(rotMat, 2, 1)
        # Gange inn rotasjonsmatrisa med rotasjonspunkt
        cornPos_ = np.dot(rot2D, cornPos.transpose())

        ###### Gjør utregninger
        pointPos_ = np.array([cornPos_[0] + pos[0], cornPos_[1] + pos[1]])
        ######

        rot2D_ = np.linalg.inv(rot2D)
        pointPos = np.dot(rot2D_, pointPos_.transpose())
        res = ndaToTupleInt(pointPos)
        return res

    pos, angle, longSide, shortSide = startCorner(box)
    moves = []

    #forskjell på startMoveX og shortMove siden første flytt
    shortMove = diameter - overlapp
    startMoveX = diameter/2 - overlapp
    startPos = [startMoveX, - aks]

    nrSideMoves = math.ceil((shortSide + overlapp)/shortMove)

    # HA med denne?:
    if shortSide < (2*startMoveX):
        if smallObjects:
            startPos = [shortSide/2, - aks]
            nrSideMoves = 1
            print("SHORT Rectangle detected")
        else:
            return 0
    ##########
    a = 0

    moves.append(startPos)
    for i in range(0, nrSideMoves):
        if a == 0:
            moves.append([moves[-1][0], moves[-1][1] + aks])
            moves.append([moves[-1][0], moves[-1][1] + longSide])
            moves.append([moves[-1][0], moves[-1][1] + aks])
            a = 1
        else:
            moves.append([moves[-1][0], moves[-1][1] - aks])
            moves.append([moves[-1][0], moves[-1][1] - longSide])
            moves.append([moves[-1][0], moves[-1][1] - aks])
            a = 0
        moves.append([moves[-1][0] + shortMove, moves[-1][1]])
    moves = moves[:-1]  # org -1, men gir ett for ekstra "move", -2 fikset problemt.
    moves_con = []
    for i in range(0, len(moves)):
        moves_con.append(calcPoint(pos, angle, moves[i]))
    return moves_con


# Tar inn optimal box i punkter. Henter lengde på sider. Finner Korteste sider
# og finner derav startpunkt som er definert som punkt som er med lavest y verdi
# på korteste side


def startCorner(box):
    points = findLines(box)
    # Indeks til de to korteste sider.
    indexList = returnTwoLowest(points, 0)
    # finne side som har den laveste y verdien.
    # Om samme y verdi, velg lavest x verdi.
    y = float('inf')
    x = float('inf')
    ind = 0
    for i in indexList:
        for j in range(1, 2):
            y1 = points[i][j][1]
            if y1 < y:
                y = y1
                x = points[i][j][0]
                ind = i
            # For tilfelle om objekt står vinkelrett med koordinatsystemet
            elif y1 == y:
                x1 = points[i][j][0]
                if x1 < x:
                    x = x1
                    y = y1
                    ind = i
    a = points[ind][1]
    b = points[ind][2]
    angle = angleDegree(a, b)
    n, d = np.shape(box)
    longSide = 0
    for i in range(0, n):
        longSide1 = points[i][0]
        if longSide1 > longSide:
            longSide = longSide1
    shortSide = points[ind][0]
    return [x, y], angle, longSide, shortSide


def angleDegree(p1, p2):
    x = p2[0] - p1[0]
    y = p2[1] - p1[1]
    return math.degrees(math.atan2(y, x))


# returnerer index plass for to minste verdier
def returnTwoLowest(box, x):  # horisontal(d) pos på array
    n, d = np.shape(box)
    lines = []
    v1, v2 = float('inf'), float('inf')
    in1, in2 = 0, 0
    for i in np.arange(0, n):
        if box[i][x] <= v1:
            v1, v2 = box[i][x], v1
            in1, in2 = i, in1
        elif box[i][x] < v2:
            v2 = box[i][x]
            in2 = i
    return in1, in2

# mar: minAreaRect
def increment_mar(mar, inc):
    mar = list(mar)
    mar[1] = list(mar[1])
    # Legg til inc på alle sidene
    mar[1][0] += inc*2
    mar[1][1] += inc*2
    mar[1] = tuple(mar[1])
    mar = tuple(mar)
    return mar
